#include <si_toolchain.h>
#include <SI_EFM8BB51_Register_Enums.h>                  // SI_SFR declarations

//-----------------------------------------------------------------------------
// Global VARIABLES
//-----------------------------------------------------------------------------
SI_LOCATED_VARIABLE_NO_INIT (reserved, uint8_t, SI_SEG_XDATA, 0x0000);

//                       G    R     B       G     R     B       G     R     B
uint8_t colors[][3] = {{0x00, 0xFF, 0x00}, {0xFF, 0x00, 0x00}, {0x00, 0x00, 0xFF},
							  {0x00, 0xFF, 0x00}, {0xFF, 0x00, 0x00}, {0x00, 0x00, 0xFF},
							  {0x00, 0xFF, 0x00}, {0xFF, 0x00, 0x00}, {0x00, 0x00, 0xFF},
							  {0x00, 0xFF, 0x00}, {0xFF, 0x00, 0x00}, {0x00, 0x00, 0xFF}};

void SiLabs_Startup (void)
{
	WDTCN = 0xDE;													// disable the watchdog
	WDTCN = 0xAD;
	CLKSEL = CLKSEL_CLKDIV__SYSCLK_DIV_1;					// set 1:1 clock divider
	while ((CLKSEL & CLKSEL_DIVRDY__BMASK) != CLKSEL_DIVRDY__READY){}	 // wait for clock to propagate thru the divider
}

void PORTS_setup(void)
{
	// P1.0  -  SCK  (SPI0), Push-Pull,  Digital
	// P1.1  -  MISO (SPI0), Open-Drain, Digital
	// P1.2  -  MOSI (SPI0), Push-Pull,  Digital
	// P1.3  -  PWM0 (CH1X), Push-Pull,  Digital
	// P1.4  -  PWM0 (CH2X), Push-Pull,  Digital
	P0 = P0_B0__LOW | P0_B0__LOW | P0_B2__LOW | P0_B3__LOW | P0_B4__LOW
			| P0_B5__LOW | P0_B6__LOW | P0_B7__LOW;

	P0MDOUT = P0MDOUT_B0__PUSH_PULL | P0MDOUT_B1__PUSH_PULL
			| P0MDOUT_B2__PUSH_PULL  | P0MDOUT_B3__PUSH_PULL
			| P0MDOUT_B4__PUSH_PULL  | P0MDOUT_B5__PUSH_PULL
			| P0MDOUT_B6__PUSH_PULL  | P0MDOUT_B7__PUSH_PULL;

	P0MDIN = P0MDIN_B0__DIGITAL | P0MDIN_B1__DIGITAL | P0MDIN_B2__DIGITAL
			| P0MDIN_B3__DIGITAL | P0MDIN_B4__DIGITAL | P0MDIN_B5__DIGITAL
			| P0MDIN_B6__DIGITAL | P0MDIN_B7__DIGITAL;

	P0SKIP = P0SKIP_B0__SKIPPED | P0SKIP_B1__SKIPPED | P0SKIP_B2__SKIPPED
			| P0SKIP_B3__SKIPPED | P0SKIP_B4__SKIPPED | P0SKIP_B5__SKIPPED
			| P0SKIP_B6__SKIPPED | P0SKIP_B7__SKIPPED;

	P1 = P1_B0__LOW | P1_B1__LOW | P1_B2__LOW | P1_B3__LOW | P1_B4__LOW
			| P1_B5__LOW | P1_B6__LOW;

	P1MDOUT = P1MDOUT_B0__PUSH_PULL | P1MDOUT_B1__PUSH_PULL
			| P1MDOUT_B2__PUSH_PULL | P1MDOUT_B3__PUSH_PULL
			| P1MDOUT_B4__PUSH_PULL  | P1MDOUT_B5__PUSH_PULL
			| P1MDOUT_B6__PUSH_PULL;

	P1MDIN = P1MDIN_B0__DIGITAL | P1MDIN_B1__DIGITAL | P1MDIN_B2__DIGITAL
			| P1MDIN_B3__DIGITAL | P1MDIN_B4__DIGITAL | P1MDIN_B5__DIGITAL
			| P1MDIN_B6__DIGITAL;

	P1SKIP = P1SKIP_B0__NOT_SKIPPED | P1SKIP_B1__NOT_SKIPPED
			| P1SKIP_B2__NOT_SKIPPED | P1SKIP_B3__NOT_SKIPPED
			| P1SKIP_B4__NOT_SKIPPED | P1SKIP_B5__SKIPPED | P1SKIP_B6__SKIPPED;

	XBR0 = XBR0_SPI0E__ENABLED;
	XBR1 = 0;
	XBR2 = XBR2_XBARE__ENABLED + XBR2_PWME__PWM0CH01 + XBR2_WEAKPUD__PULL_UPS_DISABLED;
}

void SPI_Transmit (uint8_t gColor, uint8_t rColor, uint8_t bColor)
{
	SPI0CN0 = SPI0CN0_SPIEN__ENABLED;
	P0_B4 = 1;														// enable clock propagation via CLU3
	SPI0DAT = gColor;
	SFRPAGE = 0x10;
	PWMCFG1 = PWMCFG1_PWMEN__ENABLE;
	SFRPAGE = 0x0;
   while (!SPI0CN0_SPIF){}
   SPI0CN0 = 0;

   SPI0CN0_SPIEN = 1;
   SPI0DAT = rColor;
   while (!SPI0CN0_SPIF){}
   SPI0CN0 = 0;

   SPI0CN0_SPIEN = 1;											// re-enable SPI0 to zero the bit counter
   SPI0DAT = bColor;
   while (!SPI0CN0_SPIF){}                    			// wait for the transmission end
   P0_B4 = 0;														// disable PWM propagation
   SFRPAGE = 0x10;
   PWMCFG1 = 0;
   SFRPAGE = 0x0;
   SPI0DAT = 0;
   SPI0CN0 = 0;
}

void SPI0_setup(void)											// configure SPI0 for 3-wire mode
{
	SPI0CKR = 14;													// set clock = 800 kHz (for master mode)
	SPI0CFG = SPI0CFG_CKPHA__DATA_CENTERED_SECOND;		// master mode 1
	SFRPAGE = 0x20;
	SPI0FCN1 = 0;													// discard received bytes
	SPI0PCF = SPI0PCF_SCKSEL__CLU3;							// get clock form CLU3
	SFRPAGE = 0x0;
	SPI0DAT = 0;
}

void PWM_setup(void)
{
	SFRPAGE = 0x10;
	PWMCFG0 = PWMCFG0_SYNCUPD__CH0CH1 + PWMCFG0_PWMCLKEN__ENABLE;
	PWMCPL0 = 20;
	PWMCPH0 = 0;
	PWMCPUDL0 = 20;												// signal_1 duration (~800ns)
	PWMCPUDH0 = 0;
	PWMCPL1 = 10;
	PWMCPH1 = 0;
	PWMCPUDL1 = 10;												// signal_0 duration (~400ns)
	PWMCPUDH1 = 0;
	PWMLIML = 30; 													// PWM period ~800 KHz
	PWMLIMH	= 0;
	PWMCFG2 = PWMCFG2_CNTRZERO__ZERO + PWMCFG2_CH0EN__ENABLE + PWMCFG2_CH1EN__ENABLE;
	SFRPAGE = 0x0;
}

void CLU_setup(void)
{
	SFRPAGE = 0x20;
	CLU0MX = CLU0MX_MXA__CLU0A14 + CLU0MX_MXB__CLU0B12;
	CLU0FN = 0x20;													// out = PWM.1 & n(SDO) & CLK
	CLU0CF = CLU0CF_OUTSEL__LUT;

	CLU1MX = CLU1MX_MXA__CLU1A13 + CLU1MX_MXB__CLU1B10;
	CLU1FN = 0xEA;													// out = CLU0 OR (CLK & SDO)
	CLU1CF = CLU1CF_OUTSEL__LUT + CLU1CF_OEN__ENABLE; 	// out -> P0.5

	CLU3MX = CLU3MX_MXA__CLU3A13 + CLU3MX_MXB__CLU3B10;
	CLU3FN = 0xC0;													// out = PWM.0 & P0.4
	CLU3CF = CLU3CF_OUTSEL__LUT + CLU3CF_OEN__ENABLE;	// out -> P1.5

	CLEN0 = CLEN0_C0EN__ENABLE + CLEN0_C1EN__ENABLE + CLEN0_C3EN__ENABLE;
	SFRPAGE = 0x0;
}

//-----------------------------------------------------------------------------
// main() Routine
// ----------------------------------------------------------------------------
int main (void)
{
	uint16_t i;
	PORTS_setup();
	SPI0_setup();
	CLU_setup();
	PWM_setup();

	for (i=0; i<10000; i++){}									// power-up delay to reset WS2812

   for (i=0; i<12; i++) {										// load colors
   	SPI_Transmit(colors[i][0], colors[i][1], colors[i][2]);
   }

	while (1) 														// main app loop
	{
		PCON0 = 1;													// enter Idle mode
	}
}
